package LTSP_utils::ConfigItem;

use 5.012003;
use strict;
use warnings;
use Moose;
use LTSP_utils::ImportFile;

our $VERSION = '0.01';

has 'configFile' => (
	is => 'rw',
	isa => 'Str',
	required => '1',
);
has 'rootTag' => (
	is => 'rw',
	isa => 'Str',
	default => 'NBDROOT',
);
has 'configTag' => (
	is => 'rw',
	isa => 'Str',
	default => 'CONF',
);
has 'root' => (
	is => 'rw',
	isa => 'Str|Undef',
	lazy => '1',
	builder => '_read_root',
);
has 'configList' => (
	is => 'rw',
	isa => 'ArrayRef|Undef',
	lazy => '1',
	builder => '_read_configList',
);

sub addImportFile {
	my ($self, @params) = @_;
	push(@{$self->configList}, LTSP_utils::ImportFile->new( $params[0], $params[1], $params[2])) unless @params != 3;
}

sub delImportFile {
	my ($self, @params) = @_;
	return 0 unless @params == 3;
	for my $i (0..$#{$self->configList}) {
		if (${$self->configList}[$i]->src eq $params[0] and ${$self->configList}[$i]->dst eq $params[1] and ${$self->configList}[$i]->ip eq $params[2]) {
			splice(@{$self->configList}, $i, 1);
			last;
		}
	}
}

sub hasItem {
	my ($self, @params) = @_;
	return 0 unless @params == 3;
	for my $i (0..$#{$self->configList}) {
		if (${$self->configList}[$i]->src eq $params[0] and ${$self->configList}[$i]->dst eq $params[1] and ${$self->configList}[$i]->ip eq $params[2]) {
			return 1;
		}
	}
	return 0;
}

sub write {
	my $self = shift;
	open my $file, ">", $self->configFile or die "Error at ".__PACKAGE__.": ".$!;
	print $file $self->rootTag."=".$self->root."\n";
	my $config = $self->configTag."=";
	for my $i (0..$#{$self->configList}) {
		$config = $config.${$self->configList}[$i]->src.";".${$self->configList}[$i]->dst.";".${$self->configList}[$i]->ip;
		$config = $config."," unless $i == $#{$self->configList};
	}
	print $file $config."\n";
	close $file or die "Error at ".__PACKAGE__.": ".$!;
}

around BUILDARGS => sub {
	my $orig  = shift;
	my $class = shift;

	if ( @_ == 1 && !ref $_[0] ) {
		return $class->$orig( configFile => $_[0] );
	}
	else {
		return $class->$orig(@_);
	}
};

sub BUILD {
	my $self = shift;
	die "Cannot create ".__PACKAGE__ if not defined $self->root or not defined $self->configList;
}

sub _read_root {
	my $self = shift;
	open my $file, "<", $self->configFile or die "Error: ".$!;
	my @contents = <$file>;
	close $file or die "Error: ".$!;
	my $pattern = $self->rootTag;
	for my $i (0..$#contents) {
		chomp $contents[$i];
		return $1 if ($contents[$i] =~ m/$pattern=(.*)/); 
		return undef if ($i == $#contents); 
	}
	# TODO: Generate ROOT config from sysconfig configuration.
}

sub _read_configList {
	my $self = shift;
	my $ArrayRef = ( );
	open my $file, "<", $self->configFile or die "Error at ".__PACKAGE__.": ".$!;
	my @contents = <$file>;
	close $file or die "Error at ".__PACKAGE__.": ".$!;
	my $pattern = $self->configTag;
	for my $i (0..$#contents) {
		chomp $contents[$i];
		if ($contents[$i] =~ m/$pattern=(.*)/) {
			my $itemsRef = _parse_files_config($1);
			foreach (@$itemsRef) {
				push(@$ArrayRef, LTSP_utils::ImportFile->new( ${$_}[0], ${$_}[1], ${$_}[2]));
			}
			last;
		}
		return undef if ($i == $#contents); 
	}
	return $ArrayRef;
}

sub _parse_files_config {
	my $string = shift;
	my @result = ( );
	foreach (split(/,/,$string)) {
		push(@result, [ split(/;/,$_) ]);
	}
	return \@result;
}

__PACKAGE__->meta->make_immutable;

1;

__END__

=encoding utf8

=head1 NAME

LTSP_utils::ConfigItem - Perl extension for store the configuration of the given config file from a LTSP 5 infrastructure.

=head1 SYNOPSIS

It is mandatory to provide the config file path at the object creation.
This example shows how to use the class:

use ConfigItem;

print "*** ConfigItem TEST ***\n\n";
print "*** Setting up new test file (/srv/tftpboot/KIWI/config.test) ***\n\n";
open my $file, ">", '/srv/tftpboot/KIWI/config.test' or die $!;
print $file 'NBDROOT="192.168.10.1;20000"
CONF=/KIWI/lts.conf;/etc/lts.conf;192.168.10.1';
close $file or die $!;
print " ** Creation **\n";
print " * New object with parameter '/srv/tftpboot/KIWI/config.test'\n";
my $it = ConfigItem->new('/srv/tftpboot/KIWI/config.test');
print " *  String attributes: ".$it->configFile." ".$it->rootTag." ".$it->configTag."\n";
print " *  Dynamic attribute root: ".$it->root."\n";
print " *  Dynamic attribute configList has ".@{$it->configList}." elements: \n";
foreach (@{$it->configList}) {
	print " *  ".$_->src." ".$_->dst." ".$_->ip."\n";
}
print " ** Creation end **\n\n";
print " ** Modification **\n";
print " * Adding ImportFile object with values ('/KIWI/hosts', '/etc/hosts', '192.168.10.1')\n";
$it->addImportFile('/KIWI/hosts', '/etc/hosts', '192.168.10.1');
print " * Adding ImportFile object with values ('/KIWI/resolv.conf', '/etc/resolv.conf', '192.168.10.1')\n";
$it->addImportFile('/KIWI/resolv.conf', '/etc/resolv.conf', '192.168.10.1');
print " * Deleting ImportFile object with values ('/KIWI/resolv.conf', '/etc/resolv.conf', '192.168.10.1')\n";
$it->delImportFile('/KIWI/resolv.conf', '/etc/resolv.conf', '192.168.10.1');
print " *  Dynamic attribute configList has ".@{$it->configList}." elements: \n";
foreach (@{$it->configList}) {
	print " *   ".$_->src." ".$_->dst." ".$_->ip."\n";
}
print " ** Modification end **\n\n";
print " ** Modification end **\n\n";
print " ** Querying **\n";
print " * Querying for ImportFile object ('/KIWI/hosts', '/etc/hosts', '192.168.10.1')\n";
( $it->hasItem('/KIWI/hosts', '/etc/hosts', '192.168.10.1') ) ? print " *  ImportFile found!\n" : print " *  ImportFile NOT found!\n";
print " ** Querying end **\n\n";
print " ** Write to file **\n";
print " * Calling write()\n";
$it->write();
print " ** Write to file end **\n\n";

=head1 DESCRIPTION

This package provides a way to deal with configuration files named B<config.SOMETHING>, it lets you to read, write and modify those files. 

=head2 ATRIBUTES

=head3 configFile

Stores the path of the config file. This attribute is mandatory on the class creation.

=head3 rootTag

Stores the name of the tag that defines from where the files are downloaded.

=head3 configTag

Stores the name of the tag that contains the list of files.

=head3 root

Stores the value of rootTag.

=head3 configList

Is an Array reference of a list containig ImportFile elements from values of tag configTag.
Each ImportFile element is initiaized with each comma-separated value.

=head2 EXPORT

=head3 addImportFile($src,$dst,$ip)

This method adds an ImportFile object to the B<configList> with given parameters as B<source file>, B<destination file> and B<IP address>.

Does nothing if the number of parameters are wrong.

=head3 delImportFile($src,$dst,$ip)

This method deletes first occurrence of the ImportFile object from the B<configList> that contains the values specified by the parameters.

Does nothing if that ImportFile not exists or if the number of parameters are wrong.

=head3 hasItem($src,$dst,$ip)

This method returns 1 if the B<configList> contains an B<ImportFile> with B<$src>, B<$dst>, B<$ip> values. Returns 0 otherwise.

=head3 write()

This method overwrites the source file with the current values.

=head1 SEE ALSO

LTSP 5 documentation (http://www.ltsp.org/).

=head1 AUTHOR

Jordi "Txor" Casas Ríos, E<lt>jcasas@opentrends.netE<gt>, openTrends Solucions i Sistemes, S.L.

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2011 Generalitat de Catalunya 

This file is part of Linkatedu_11_4.  

Linkatedu_11_4 is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 3 of the License, or (at your option) any
later version.

Linkatedu_11_4 is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for further
details, currently published at http://www.gnu.org/licenses/gpl.txt or in the
gpl.txt in the ["legal" folder] of this distribution.

You should have received a copy of the GNU General Public License along with
this program; If not, see <http://www.gnu.org/licenses/>

This library is free software; you can redistribute it and/or modify it under the same terms as Perl itself.

=cut
