package LTSP_utils::ConfigList;

use 5.012003;
use strict;
use warnings;
use Moose;
use LTSP_utils::ConfigItem;
use Try::Tiny;

our $VERSION = '0.01';

has 'fileType' => (
	is => 'rw',
	isa => 'Str',
	default => 'config.',
);
has 'path' => (
	is => 'rw',
	isa => 'Str',
	default => '/srv/tftpboot/KIWI'
);
has 'list' => (
	is => 'rw',
	isa => 'ArrayRef',
	lazy => '1',
	builder => '_load_list',
);

sub addConfig {
	my ($self, $file) = @_;
	return 0 unless defined $file;
	return 0 unless not $self->_exist($file);
	my $new_item = LTSP_utils::ConfigItem->new($self->path."/".$self->fileType."default");
	$new_item->configFile($self->path."/".$file);
	$new_item->write();
	push(@{$self->list}, $new_item);
}

sub delConfig {
	my ($self, $file) = @_;
	return 0 unless defined $file;
	for my $i (0..$#{$self->list}) {
		if (${$self->list}[$i]->configFile() eq $self->path."/".$file) {
			unlink ${$self->list}[$i]->configFile() or die "Error at ".__PACKAGE__.": ".$!;
			splice(@{$self->list}, $i, 1);
			last;
		}
	}
}

sub modifyConfig {
	my ($self, $file, @modification) = @_;
	return 0 unless defined $file;
	return 0 unless @modification == 4;
	return 0 unless $modification[0] eq 'ADD' or $modification[0] eq 'DEL';
	return 0 unless not $self->_exist($file);
	for my $i (0..$#{$self->list}) {
		if (${$self->list}[$i]->configFile() eq $self->path."/".$file) {
			if ($modification[0] eq 'ADD') {
				${$self->list}[$i]->addImportFile($modification[1], $modification[2], $modification[3]);
			}
			elsif ($modification[0] eq 'DEL') {
				${$self->list}[$i]->delImportFile($modification[1], $modification[2], $modification[3]);
			}
			else { 
				return 0; 
			}
			${$self->list}[$i]->write();
			last;
		}
	}
}

sub queryConfig {
	my ($self, $file, @values) = @_;
	return 0 unless defined $file;
	return 0 unless @values == 3;
	return 0 unless not $self->_exist($file);
	for my $i (0..$#{$self->list}) {
		if (${$self->list}[$i]->configFile() eq $self->path."/".$file) {
			return 1 if ${$self->list}[$i]->hasItem( $values[0], $values[1], $values[2] );
		}
	}
	return 0;
}

sub listConfig {
	my $self = shift;
	my @names;
	foreach (@{$self->list}) {
		push(@names, $_->configFile);
	}
	return @names;
}

sub _load_list {
	my $self = shift;
	my $ArrayRef = ( );
	my $pattern = $self->fileType();
	opendir my $dir, $self->path() or die "Error at ".__PACKAGE__.": ".$!;
	while (readdir $dir) {
		chomp;
		next if (not $_ =~ m/^$pattern/);
		try {
			push(@$ArrayRef, LTSP_utils::ConfigItem->new($self->path."/".$_));
		}
	}
	closedir $dir;
	return $ArrayRef;
}

sub _exist {
	my ($self, $file) = @_;
	foreach (@{$self->list}) {
		return 1 if $_->configFile eq $file;
	}
	return 0;
}

1;
__END__

=encoding utf8

=head1 NAME

LTSP_utils::ConfigList - Perl extension for mantain a list of B<ConfigItem>s allowing you to manage them.

=head1 SYNOPSIS

This example shows how to use the class:

use ConfigList;

print "*** ConfigList TEST ***\n\n";
print "*** Clearing up test files at /srv/tftpboot/KIWI/ as config.test1 config.test2 config.test3 ***\n\n";
unlink '/srv/tftpboot/KIWI/config.test1' or die $! if -f '/srv/tftpboot/KIWI/config.test1';
unlink '/srv/tftpboot/KIWI/config.test2' or die $! if -f '/srv/tftpboot/KIWI/config.test2';;
unlink '/srv/tftpboot/KIWI/config.test3' or die $! if -f '/srv/tftpboot/KIWI/config.test3';
print " ** Creation **\n";
print " * New object\n";
my $list = ConfigList->new();
print " *  String attributes: ".$list->fileType()." ".$list->path()."\n";
print " *  Dynamic attribute list has ".@{$list->list()}." elements: \n";
foreach (@{$list->list()}) {
print " *   ".$_->configFile()."\n";
}
print " ** Creation end **\n\n";
print " ** Modification **\n";
print " * Adding new config file config.test1\n";
$list->addConfig('config.test1');
print " * Adding new config file config.test2\n";
$list->addConfig('config.test2');
print " * Adding new config file config.test3\n";
$list->addConfig('config.test3');
print " * Deleting config file config.test3\n";
$list->delConfig('config.test3');
print " * Adding triad ('/KIWI/added.conf', '/etc/added.conf', '192.168.10.1') to config.test1\n";
my @mod1 = qw(ADD /KIWI/added.conf /etc/added.conf 192.168.10.1);
$list->modifyConfig('config.test1', @mod1);
print " * Deleting triad ('/KIWI/lts.conf', '/etc/lts.conf', '192.168.10.1') from config.test2\n";
my @mod2 = qw(DEL /KIWI/lts.conf /etc/lts.conf 192.168.10.1);
$list->modifyConfig('config.test2', @mod2);
print " ** Modification end **\n\n";
print " ** Querying **\n";
print " * Querying for triad ('/KIWI/added.conf', '/etc/added.conf', '192.168.10.1') at config.test1\n";
my @query = qw(/KIWI/added.conf /etc/added.conf 192.168.10.1);
( $list->queryConfig( 'config.test1', @query ) ) ? print " *  Triad found!\n" : print " *  Triad NOT found!\n";
print " ** Querying end **\n\n";
print " ** List ConfigItems **\n";
print " * Calling listConfig() and printing it:\n";
foreach ($list->listConfig()) {
print " *  ".$_."\n";
}
print " ** List ConfigItems end **\n\n";

=head1 DESCRIPTION

This package provides a way to mantain a list of B<config.SOMETHING> files using the class B<ConfigItem>. Allows you to create new files and to delete or modify existing ones.

It is not fully implemented, by now provides a way to add import files and to delete existing configurations.

=head2 ATRIBUTES

=head3 fileType

Stores the prefix of the config file name.

=head3 path

Stores the working path.

=head3 list

Stores the array reference of the list of B<ConfigItems>.

=head2 EXPORT

=head3 addConfig($file)

This method adds a B<ConfigItem> with file name B<$file> to the class atribute B<list>. First copies it from the file B<config.default>, then changes its file name to the new name and writes a new file.

Does nothing and returns 0 if the number of parameters are wrong or the B<$file> already exists on the B<list>.

=head3 delConfig($file)

This method deletes the first occurrence of the B<ConfigItem> identified by B<$file> from the class atribute B<list>. Also deletes the asociated file.

Does nothing if the number of parameters are wrong or the B<$file> don't exists on the B<list>.

=head3 modifyConfig($file,@modification)

This method modifies the first occurrence of an existing B<ConfigItem> identified by B<$file> from the class atribute B<list>. The B<@modification> is an array of a modification description.

B<@modification> definition if not fully implemented. By now the the number of elements is fixed and it's 4, the first can only be B<ADD> or B<DEL> and the other three are a triad of B<source file>, B<destination file> and B<IP address>.

Does nothing if any of the parameters are wrong or the B<$file> don't exists on the B<list>.

=head3 queryConfig($file,@values)

This method returns 1 if the selected ConfigItem described with B<$file> contains an B<ImportFile> with the values in B<@values>. Returns 0 otherwise.

B<@values> must contain three elements as B<ImportFile> elements.

Does nothing if any of the parameters are wrong or the B<$file> don't exists on the B<list>.

=head3 listConfig()

This method returns an array with all file names of B<ConfigItem>s from the class atribute B<list>.

=head1 SEE ALSO

LTSP 5 documentation (http://www.ltsp.org/).

=head1 AUTHOR

Jordi "Txor" Casas Ríos, E<lt>jcasas@opentrends.netE<gt>, openTrends Solucions i Sistemes, S.L.

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2011 Generalitat de Catalunya 

This file is part of Linkatedu_11_4.  

Linkatedu_11_4 is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 3 of the License, or (at your option) any
later version.

Linkatedu_11_4 is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for further
details, currently published at http://www.gnu.org/licenses/gpl.txt or in the
gpl.txt in the ["legal" folder] of this distribution.

You should have received a copy of the GNU General Public License along with
this program; If not, see <http://www.gnu.org/licenses/>

This library is free software; you can redistribute it and/or modify it under the same terms as Perl itself.

=cut
